using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

using MacGruber;

public class VariableTrace : MVRScript
{
    private JSONStorableBool myAcceptTrigger;

    private FloatTrigger myFloatTrigger;

    private JSONStorableFloat variable_float = null;

    private JSONStorableFloat magnification = null;

    string instance;
    private JSONStorableStringChooser atom_chooser;
    private JSONStorableStringChooser plugin_chooser;
    private JSONStorableStringChooser variable_chooser;

    private List<object> ui_objects = new List<object>();

    public override void Init()
    {
        SimpleTriggerHandler.LoadAssets();

        generate_uiplugin();
    }

    private void Update()
    {
        if (this.instance == "JSONStorableFloat")
            this.variable_float.val =
                SuperController.singleton
                    .GetAtomByUid(this.atom_chooser.val)
                    .GetStorableByID("plugin#" + this.plugin_chooser.val)
                    .GetFloatParamValue(this.variable_chooser.val) * this.magnification.val;
    }

    private void generate_uiplugin()
    {
        Utils.OnInitUI(CreateUIElement);

        Utils.SetupInfoText(
            this,
            "<color=#000><size=35><b>VariableTrace</b></size></color>\n\n"
                + "<color=#333>"
                + "This is a simple plugin can get other plugin float parameter "
                + "and trigger to other event.\n\n"
                + "The 'magnification' can enlarge or reduce the trigger value.\n"
                + "</color>",
            300.0f,
            true
        );
        myAcceptTrigger = Utils.SetupToggle(this, "AcceptTrigger", true, false);

        myFloatTrigger = new FloatTrigger(this, "OnTrigger");
        Utils.SetupButton(this, "Actions OnTrigger", myFloatTrigger.OpenPanel, false);

        SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;

        this.magnification = new JSONStorableFloat("Magnification", 1f, 0f, 10f, false, true);
        RegisterFloat(this.magnification);
        CreateSlider(this.magnification, false);

        Utils.SetupInfoOneLine(this, "<size=30><b>Select Parameter</b></size>", false);

        Utils.SetupSpacer(this, 3, false);

        this.atom_chooser = new JSONStorableStringChooser(
            "Selected Atom",
            new List<string>(),
            string.Empty,
            "Selected Atom",
            (string v) =>
            {
                update_plugin_chooser(v);
            }
        );
        RegisterStringChooser(this.atom_chooser);
        CreateFilterablePopup(this.atom_chooser, false);

        this.plugin_chooser = new JSONStorableStringChooser(
            "Selected Plugin",
            new List<string>(),
            string.Empty,
            "Selected Plugin",
            (string v) =>
            {
                update_variable_chooser(v);
            }
        );
        RegisterStringChooser(this.plugin_chooser);
        CreateFilterablePopup(this.plugin_chooser, false);

        this.variable_chooser = new JSONStorableStringChooser(
            "Selected Variable",
            new List<string>(),
            string.Empty,
            "Selected Variable",
            (string v) =>
            {
                restore_uiplugin();
            }
        );
        RegisterStringChooser(this.variable_chooser);
        CreateFilterablePopup(this.variable_chooser, false);

        update_atom_chooser();
    }

    void update_atom_chooser()
    {
        string uid = "";
        List<string> uids = SuperController.singleton.GetAtomUIDs();
        foreach (string u in uids)
            if (SuperController.singleton.GetAtomByUid(u).type == "Person")
                uid = u;

        this.atom_chooser.val = uid;
        this.atom_chooser.choices = null;
        this.atom_chooser.choices = uids;
        this.atom_chooser.displayChoices = uids;
    }

    void update_plugin_chooser(string name)
    {
        var choices = new List<string>();

        foreach (var n in SuperController.singleton.GetAtomByUid(name).GetStorableIDs())
        {
            if (!n.StartsWith("plugin#"))
                continue;
            choices.Add(n.Replace("plugin#", ""));
        }

        this.plugin_chooser.val = choices.First();
        this.plugin_chooser.choices = null;
        this.plugin_chooser.choices = choices;
        this.plugin_chooser.displayChoices = choices;
    }

    void update_variable_chooser(string name)
    {
        var choices = new List<string>();
        var plugin = SuperController.singleton
            .GetAtomByUid(this.atom_chooser.val)
            .GetStorableByID("plugin#" + name);
        foreach (var v in plugin.GetAllParamAndActionNames())
            if (
                v != null
                && !v.StartsWith("SaveToStore")
                && !v.StartsWith("RestoreAllFrom")
                && !v.StartsWith("RestorePhysic")
                && !v.StartsWith("RestoreAppearance")
            )
                try
                {
                    var instance = plugin.GetParam(v).ToString();
                    if (instance == "JSONStorableFloat"
                    // || instance == "JSONStorableBool"
                    // || instance == "JSONStorableString"
                    )
                    {
                        choices.Add(v);
                        // SuperController.LogMessage(v + $" instance " + this.instance.val);
                    }
                }
                catch { }

        // this.variable_chooser.val = choices.First();
        this.variable_chooser.choices = null;
        this.variable_chooser.choices = choices;
        this.variable_chooser.displayChoices = choices;
    }

    private void restore_uiplugin()
    {
        var choices = new List<string>();
        this.instance = SuperController.singleton
            .GetAtomByUid(this.atom_chooser.val)
            .GetStorableByID("plugin#" + this.plugin_chooser.val)
            .GetParam(this.variable_chooser.val)
            .ToString();

        // this.instance.val = instance;
        Utils.RemoveUIElements(this, ui_objects);
        if (instance == "JSONStorableFloat")
        {
            this.variable_float = new JSONStorableFloat(
                "Variable",
                0f,
                (float v) =>
                {
                    Trigger(v);
                },
                0f,
                1f,
                false,
                true
            );
            ui_objects.Add(CreateSlider(this.variable_float, true));
        }
    }

    private void Trigger(float v)
    {
        if (myAcceptTrigger.val)
            myFloatTrigger.Trigger(v);
    }

    private void OnAtomRename(string oldid, string newid)
    {
        myFloatTrigger.SyncAtomNames();
    }

    public override JSONClass GetJSON(
        bool includePhysical = true,
        bool includeAppearance = true,
        bool forceStore = false
    )
    {
        JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
        if (includePhysical || forceStore)
        {
            needsStore = true;
            jc[myFloatTrigger.Name] = myFloatTrigger.GetJSON(base.subScenePrefix);
        }
        return jc;
    }

    public override void LateRestoreFromJSON(
        JSONClass jc,
        bool restorePhysical = true,
        bool restoreAppearance = true,
        bool setMissingToDefault = true
    )
    {
        base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
        if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
        {
            myFloatTrigger.RestoreFromJSON(
                jc,
                base.subScenePrefix,
                base.mergeRestore,
                setMissingToDefault
            );
        }
    }

    private void OnDestroy()
    {
        SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
        myFloatTrigger.Remove();
    }
}
